!!$=head1 NAME
!!$
!!$Decon - Deconvolution (N-dimensional)
!!$
!!$=head1 SYNOPSIS
!!$
!!$Decon < data.H filt= predictive=0 > decon.H 
!!$
!!$=head1 DESCRIPTION
!!$
!!$Deconvolution: predictive, Lomoplan, steep dip. 
!!$Uses the helix and patching technology.
!!$
!!$=head1 PARAMETERS
!!$
!!$=over 3
!!$
!!$=item from history
!!$
!!$B<integer array> I<n> - I<n1>, I<n2>, I<n3>, etc
!!$
!!$=item from par
!!$
!!$B<filename> I<filt> - helix-type local PEF
!!$
!!$B<logical> I<predictive>=0 - predictive deconvolution
!!$
!!$B<integer> I<rect1> (optional) - smoothing on the first axis
!!$
!!$=item from aux (filt)
!!$
!!$B<integer> I<dim> - number of dimensions
!!$
!!$B<integer array> I<w> - patch size
!!$
!!$B<integer array> I<k> - number of windows
!!$
!!$=back
!!$
!!$=head1 EXAMPLES
!!$
!!$F<gee/lmn/sep7390/>, F<gee/lmn/sep7790>, 
!!$F<gee/pch/ida90>, F<gee/pch/steep90>
!!$
!!$=head1 MODULES
!!$
!!$F<tent.r90>, F<patching.r90>, F<loconvol.r90> , F<helix.r90>
!!$F<triangle.r90>
!!$
!!$=head1 SEE ALSO
!!$
!!$L<Lopef>, L<Helicon>
!!$
!!$=cut
!%
program Decon
  use sep
  use helix
  use tent
  use patching
  use loconvol
  use triangle

  implicit none
  integer                                   :: n123, n1,i,ik, dim,nk,na, r1, st
  integer,       dimension (:), pointer     :: n, w, k, a, center
  real,          dimension (:), allocatable :: data, wind, resi, sign
  type( filter), dimension (:), pointer     :: aa
  type (filter)                             :: bb
  real                                      :: signvar, datavar
  logical  	                            :: predictive

  call sep_init (SOURCE)
  call from_history (n123, compress = .true.)

  call from_aux ("filt","dim", dim)
  allocate (n (dim), w (dim), k (dim), a (dim), center (dim))
  call from_history ("n",n)
  n1 = product (n)

  call from_aux ("filt", "w", w)
  call from_aux ("filt", "k", k); nk = product (k)
  call from_aux ("filt", "a", a)
  call from_aux ("filt", "center", center)

  call from_aux ("filt", na)
  call allocatehelix (bb, na)
  call from_aux ("filt", "lag", bb%lag)
  call from_par ("predictive", predictive, .false.)
  call from_par ("rect1", r1, 0)
  call sep_close ()
  allocate (data (n123), resi (n123), sign (n123))
  call sep_read (data)
  data = data / maxval (abs (data))

  allocate (aa (nk))

  do ik = 1, nk
     allocate ( aa(ik)%flt (na))
     aa(ik)%lag => bb%lag
     nullify (aa(ik)%mis)
  end do

  allocate (wind (product (w)))
  call tentn (w, center, a, wind) 
  do i = 1, n123-n1+1, n1
     call loconvol_init (aa)
     do ik = 1, nk     
        call sep_read (aa(ik)%flt,"filt") 
     end do
     call patchingn (loconvol_lop, data(i:i+n1-1), resi(i:i+n1-1), &
     k, n, w, wind)
  end do

  if (predictive) then
     sign = data - resi
     signvar = sqrt (sum (sign * sign))
     datavar = sqrt (sum (data * data))
     where (resi == 0.) sign = data * signvar / datavar
     call sep_write (sign)
  else if (r1 > 0) then
     call triangle_init (r1, n (1))
     do i = 1, n123-n(1)+1, n(1)
        st = triangle_lop (.true., .false., &
        sign(i:i+n(1)-1), resi(i:i+n(1)-1))
     end do
     call sep_write (sign)
  else
     signvar = sqrt (sum (resi * resi))
     datavar = sqrt (sum (data * data))
     where (resi == 0.) resi = data * signvar / datavar
     call sep_write (resi)
  end if

  do ik = 1, nk
     deallocate ( aa(ik)%flt)
  end do
  call deallocatehelix (bb)
  deallocate (data, resi, sign, aa)
  deallocate (n, w, k, a, center)
  call exit (0)
end program Decon


