module slowness_1D
  
  use adj_mod

  implicit none
  
  integer, private   :: nt 
  integer, private   :: nx 
  integer, private   :: nh 
  integer, private   :: nm 
  real,    private   :: t0 
  real,    private   :: dt 
  real,    private   :: x0 
  real,    private   :: dx 
  real,    private   :: m0 
  real,    private   :: dm 
  real,    private   :: h0 
  real,    private   :: dh
  integer, private   :: verb
  real,    private, dimension(:), pointer :: s0
  
contains
  
  subroutine slowness_init ( t0_in,dt_in,nt_in,&
  &                          x0_in,dx_in,nx_in,&
  &                          m0_in,dm_in,nm_in,&
  &                          h0_in,dh_in,nh_in,&
  &                          s0_in,&
  &                          verb_in )
    
    real    :: t0_in 
    real    :: dt_in 
    real    :: x0_in  ! data
    real    :: dx_in  ! data
    real    :: m0_in  ! model
    real    :: dm_in  ! model
    real    :: h0_in  
    real    :: dh_in  
    
    integer :: nt_in 
    integer :: nx_in  ! data
    integer :: nm_in  ! model  
    integer :: nh_in  
    integer :: verb_in
 
    real, dimension(:) :: s0_in
    allocate(s0(nt_in))
    
    t0 = t0_in
    dt = dt_in 
    nt = nt_in
    x0 = x0_in 
    dx = dx_in
    nx = nx_in
    m0 = m0_in 
    dm = dm_in 
    nm = nm_in
    h0 = h0_in
    dh = dh_in
    nh = nh_in
    s0 = s0_in
    verb = verb_in

  end subroutine slowness_init

  subroutine slowness_close()
    if (allocated(s0)) deallocate(s0)
  end subroutine slowness_close

  function slowness_lop(adj,add,modl,data) result(stat)
    
    integer              :: stat
    logical,intent(in)   :: adj,add
    real,dimension(:)    :: modl
    real,dimension(:)    :: data
    call adjnull   (adj,add,modl,data)
    call slowness_lop1 (adj,modl,data)
    stat=0

  end function slowness_lop
 
  subroutine slowness_lop1  (adj,modl,data)
    
    logical,intent(in)       :: adj
    real,dimension(nt,nm)    :: modl
    real,dimension(nt,nh,nx) :: data
    integer                  :: ix,it,ih,itm,ip,im
    real                     :: tau0, cmp0, off, offm, offp, taum, slom, slop, alpha, delta_x

    ! Span data space
    cmp: do ix=1,nx

       tau_data: do it=1,nt
          
          tau0 = (it-1)*dt+t0 ! "depth" location
          cmp0 = (ix-1)*dx+x0 ! CMP location
                    
          offset: do ih=1,nh
             
             off =((ih-1)*dh+h0)/2 
             offm=cmp0-off ! CMP location - offset
             offp=cmp0+off ! CMP location + offset
            
             delta_x = off/it ! step in x

             tau_model: do itm=1,it
             
                taum = (itm-1)*dt+t0  ! Scans model depth from top to actual data depth

                if (tau0.eq.0) then
                   slom = offm        
                   slop = offp
                else
                   slom =  off*taum/tau0+offm ! Estimate the "downgoing straight ray" for a given depth
                   slop = -off*taum/tau0+offp ! Estimate the "upgoing straight ray"   for a given depth
                end if

                im = 1.5 + (slom - m0)/dm    ! Estimate the index of the surface location of the downgoing ray in the model space
                ip = 1.5 + (slop - m0)/dm    ! Estimate the index of the surface location of the   upgoing ray in the model space

                if (tau0.eq.0) then
                   alpha = 0 
                else 
                   alpha = delta_x**2*s0(itm)/sqrt(dt**2/4+(delta_x*s0(itm))**2) ! dl without sigma
                end if

                if (adj) then
                   modl(itm,im) = modl(itm,im) + alpha*data(it,ih,ix) ! Back projection along the downgoing ray
                   modl(itm,ip) = modl(itm,ip) + alpha*data(it,ih,ix) ! Back projection along the upgoing   ray
                else
                   data(it,ih,ix) = data(it,ih,ix) + alpha*modl(itm,im) + alpha*modl(itm,ip) 
                end if
                
             end do tau_model

          end do offset

       end do tau_data
    
    end do cmp
    
  end subroutine slowness_lop1

end module slowness_1D
